package com.qmth.distributed.print.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.service.DownloadService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.teachcloud.common.bean.result.TaskListResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * 下载管理
 */
@Api(tags = "系统管理-任务管理Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.data}/download")
public class DownloadController {

    @Resource
    private DownloadService downloadService;

    /**
     * 分页
     *
     * @param semesterId  学期ID
     * @param examId      考试ID
     * @param orgId 学院
     * @param courseName  课程名称
     * @param pageNumber  分页页码
     * @param pageSize    分页数
     * @return Object
     */
    @ApiOperation(value = "分页查询")
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = TaskListResult.class)})
    public Object page(@ApiParam(value = "学期ID", required = false) @RequestParam(required = false) Long semesterId,
                       @ApiParam(value = "考试ID", required = false) @RequestParam(required = false) Long examId,
                       @ApiParam(value = "学院ID", required = false) @RequestParam(required = false) Long orgId,
                       @ApiParam(value = "课程名称", required = false) @RequestParam(required = false) String courseName,
                       @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return downloadService.page(semesterId, examId, orgId, courseName, pageNumber, pageSize);
    }

    /**
     * 单个科目下载（只下载未曝光试卷）
     *
     * @param id 任务ID
     */
    @ApiOperation(value = "单课程下载")
    @RequestMapping(value = "/download_one", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "单课程下载", response = TaskListResult.class)})
    public void downloadOne(HttpServletResponse response, @ApiParam(value = "任务ID", required = false) @RequestParam(required = false) Long id) throws Exception {
        downloadService.paperDownload(response, id);
    }

    /**
     * 批量下载
     *
     * @param semesterId  学期ID
     * @param examId      考试ID
     * @param collegeName 学院
     * @param courseName  课程名称
     * @return Object
     */
    @ApiOperation(value = "批量下载")
    @RequestMapping(value = "/download_batch", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "批量下载", response = TaskListResult.class)})
    public Object downloadBatch(@ApiParam(value = "学期ID", required = false) @RequestParam(required = false) Long semesterId,
                                @ApiParam(value = "考试ID", required = false) @RequestParam(required = false) Long examId,
                                @ApiParam(value = "学院", required = false) @RequestParam(required = false) String collegeName,
                                @ApiParam(value = "课程名称", required = false) @RequestParam(required = false) String courseName) {
        return null;
    }
}
